#define IDD_CONFIG                      101
#define IDB_LOGO                        103
#define IDC_OK                          1001
#define IDC_CANCEL                      1002
#define IDC_WINDOWMODE			1003
#define IDC_SOUND                       1004
#define IDC_SCREENMODE                  1006
#define IDC_ZBUFFER                     1016
#define IDC_STATIC			-1
